/* NsmConText: Nvu Site Manager ConText extension
 *
 * This patch of the site manager opens non-html files with external editors.
 * It also integrates a Launchy menu (http://gemal.dk/mozilla/launchy.html) into N|vu's site manager.
 *
 * My main goals are:
 *    > preview html files in Firefox, IE, Opera, ...
 *    > edit code files (js, php, asp...) with an appropriate editor
 *    > html-tidy files ("pretty-print" the code)
 *    > share launchy.xml with Firefox
 * 
 * For now, there are only 3 fixed file categories:
 *    > images files (as defined in the 'IsHiddenByFilter' function)
 *    > css stylesheets (to open with cascades, not working yet)
 *    > non-html, non-image files (to open with a text editor)
 * An option box allows to choose a editor for each category.
 *
 * TODO (= planned for sometime):
 *    > modify the option box to let the user add categories (id + name + regexp + command line)
 *    > auto reload html documents after editing html or css files with external apps
 *    > open CSS files with CaScadeS
 *    > "FTP sync" menu entry (I really miss that feature!)
 *    > XUL interface for ImageMagick (wouldn't that be neat?!?)
 *
 * Known bugs (= should be corrected soon):
 *    > Launchy integration is limited (you can't pass args, MIME type is ignored, ...)
 *    > ugly code (no 'locales' yet for instance...)
 *    > this is my first XUL hack... there might be lots of bugs :-(
 */

//
// Nsm windows & prefs
var NsmPrefs = Components.classes["@mozilla.org/preferences-service;1"].
    getService(Components.interfaces.nsIPrefService).getBranch("editor.nvu.sitemanager.");
var ToolbarPrefs = Components.classes["@mozilla.org/preferences-service;1"].
    getService(Components.interfaces.nsIPrefService).getBranch("editor.toolbars.");

var NsmApps;

function openCascadesDialog() {
  // this function doesn't work yet (never called anyway)
  //~ var doc = new ??? 
  //~ alert("CaScadeS");
  window.openDialog("chrome://cascades/content/EdCssProps.xul","_blank", "chrome,close,modal,titlebar");
  window._content.focus();
  //~ alert("Done");
}

function openPrefDialog() {
  openDialog("chrome://nsmcontext/content/NsmConText.xul","Preferences","dialog=yes");
}
function openLaunchyDialog() {
  openDialog("chrome://launchy/content/options.xul","Launchy","dialog=yes");
}
function getNsmPrefs() {
	document.getElementById("NsmDefaultEditor-img").value = NsmPrefs.getCharPref("imgEditor");
	document.getElementById("NsmDefaultEditor-css").value = NsmPrefs.getCharPref("cssEditor");
	document.getElementById("NsmDefaultEditor-ext").value = NsmPrefs.getCharPref("extEditor");
	
	document.getElementById("NsmHtmlShowBrowsers").checked = NsmPrefs.getBoolPref("ShowBrowsers");
	document.getElementById("NsmHtmlShowEditors").checked  = NsmPrefs.getBoolPref("ShowEditors");
	
	document.getElementById("StructToolbar-ShowId").checked    = ToolbarPrefs.getBoolPref("ShowId");
	document.getElementById("StructToolbar-ShowClass").checked = ToolbarPrefs.getBoolPref("ShowClass");
	document.getElementById("StructToolbar-ShowAttrs").checked = ToolbarPrefs.getBoolPref("ShowAttrs");
}

function setNsmPrefs() {
	NsmPrefs.setCharPref("imgEditor", document.getElementById("NsmDefaultEditor-img").value);
	NsmPrefs.setCharPref("cssEditor", document.getElementById("NsmDefaultEditor-css").value);
	NsmPrefs.setCharPref("extEditor", document.getElementById("NsmDefaultEditor-ext").value);
	launchyContext.doInit();
	
	NsmPrefs.setBoolPref("ShowBrowsers", document.getElementById("NsmHtmlShowBrowsers").checked);
	NsmPrefs.setBoolPref("ShowEditors",  document.getElementById("NsmHtmlShowEditors").checked);
	
	ToolbarPrefs.setBoolPref("ShowId",    document.getElementById("StructToolbar-ShowId").checked);
	ToolbarPrefs.setBoolPref("ShowClass", document.getElementById("StructToolbar-ShowClass").checked);
	ToolbarPrefs.setBoolPref("ShowAttrs", document.getElementById("StructToolbar-ShowAttrs").checked);
	UpdateStructToolbar;
}

function NsmEdit(newTab, FileURL) {
	//~ var prefs = window.top.GetPrefs();
	//~ var newTab = false; // default is false
	//~ try {
		//~ newTab = prefs.getBoolPref("editor.nvu.sitemanager.openInNewTab");
		//~ DEBUG ( "prefs found" );
	//~ } catch (e) {};
	
	if (FileURL=="")
		FileURL = gFilteredItemsArray[GetSelectedItem(gDialog.SiteTree)].url;
	EnableAllUI(false);
	window.top.document.getElementById("tabeditor").endNavigationCallback = EndNavigation;
	window.top.editPage(FileURL, window.top, true, newTab);
}
//
// Modified 'openFile' and 'IsHiddenByFilter' functions (override those in 'comm.jar/editor/content/sitemanager.js')
function openFile(e) {
	if (e.button != 0)
		return;
	
	var item = gFilteredItemsArray[GetSelectedItem(gDialog.SiteTree)];
	var editor = "";
	//~ alert ("root = " + NVU_ROOT_PATH + "\nprofile = " + NVU_PROFILE_PATH);

	// Folder (do nothing)
	if (item.isContainer) {
	}
	// HTML file: open in a new / existing tab
	else if (!IsHiddenByFilter("html", item.url)) {
		var prefs = window.top.GetPrefs();
		var newTab = false; // default is false
		try {
			newTab = prefs.getBoolPref("editor.nvu.sitemanager.openInNewTab");
			DEBUG ( "prefs found" );
		} catch (e) {};
		//~ EnableAllUI(false);
		//~ window.top.document.getElementById("tabeditor").endNavigationCallback = EndNavigation;
		//~ window.top.editPage(item.url, window.top, true, newTab);
		NsmEdit(newTab, item.url)
	} 
	// other file: open with external editor
	else {
		// Get file type
		// TODO: loop with all user-defined filters
		if (!IsHiddenByFilter("images", item.url))
			editor = "imgEditor";
		else if (!IsHiddenByFilter("css", item.url))
			editor = "cssEditor";
		else
			editor = "extEditor";
		
		// Get appropriate editor path
		try {
			// ":" is used as PortableNvu's root dir (demo, should use '%NvuRoot%' and '%NvuProfile%' instead)
			var AppPath = NsmPrefs.getCharPref(editor);
			//~ DEBUG ( "prefs found" );
		} catch (e) {
			EnableAllUI(false);
			window.top.document.getElementById("tabeditor").endNavigationCallback = EndNavigation;
			window.top.document.open(item.url, window.top, true, true);
		};
		
		// Launch appropriate editor
		try {
			NsmLaunchExe(AppPath, item.url);
			//~ DEBUG ( "app launched" );
		} catch (e) {};
	}
}
function IsHiddenByFilter(filter, fileName) {
	if (filter == "all")
		return false;
	
	if (filter == "html")
		var re = /\.html?$|\.shtml?$/i ;
	else if (filter == "images")
		re = /\.gif$|\.png$|\.jpg$|\.jpeg$|\.ico$/i ;
	else if (filter == "css")
		re = /\.css$/i ;
	
	return !re.test(fileName);
}
//
// Modified 'loadExternalURL' function (override that in 'comm.jar/editor/content/ComposerCommands.js')
var NsmPreviewAppRef = -1;
function loadExternalURL(url) {
	if (url == "")
		url = gFilteredItemsArray[GetSelectedItem(gDialog.SiteTree)].url;
	if (url) {
		if (NsmPreviewAppRef >= 0) { // launch custom browser
			//~ var AppPath = LaunchyContext.apps[NsmPreviewAppRef].exe;
			NsmLaunch(NsmPreviewAppRef, url);
			NsmPreviewAppRef = -1; // reset to default browser
		} else { // launch default browser
			var ioService = Components.classes["@mozilla.org/network/io-service;1"]
				.getService(Components.interfaces.nsIIOService);
			var uri = ioService.newURI(url, null, null);
			var extProtocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
				.getService(Components.interfaces.nsIExternalProtocolService);
			extProtocolSvc.loadUrl(uri);
		}
	}
}

//
// Modified 'UpdateStructToolbar' function (override that in 'comm.jar/editor/content/editor.js')
function newStructToolbarButton(tag, attr) {
	var ttip, tagId, attrName, attrValue, attref;
	
	button = document.createElementNS(XUL_NS, "toolbarbutton");
	button.setAttribute("value", tag);
	
	// get user prefs
	var ShowId    = false;
	var ShowClass = false;
	var ShowAttrs = false;
	try {
		ShowId    = ToolbarPrefs.getBoolPref("ShowId");
		ShowClass = ToolbarPrefs.getBoolPref("ShowClass");
		ShowAttrs = ToolbarPrefs.getBoolPref("ShowAttrs");
	} catch (e) {};
	
	// Let's add "id" and "class" attributes in the label
	// and other attributes in the tooltip
	ttip = "";
	tagId = "";
	for (attref = 0; attref < attr.length; attref++) {
		attrName  = attr[attref].nodeName;
		attrValue = attr[attref].value;
		if (ShowId && (attrName == "id"))
			tagId = ' id="' + attrValue + '"' + tagId;
		else if (ShowClass && (attrName == "class"))
			tagId += ' class="' + attrValue + '"';
		else if (ShowAttrs)
			ttip += attrName + '="' + attrValue + '" ';
	}
	button.setAttribute("label", "<" + tag + tagId + ">");
	
	if (ttip != "")
		button.setAttribute("tooltiptext", ttip);
	
	return button;
}
function UpdateStructToolbar() {
  var editor = GetCurrentEditor();
  if (!editor) return;

  var mixed = GetSelectionContainer();
  if (!mixed) return;
  var element = mixed.node;
  var oneElementSelected = mixed.oneElementSelected;

  if (!element) return;

  NotifyProcessors(kProcessorsWhenSelectionChanges, element);

  if (element == gLastFocusNode &&
      oneElementSelected == gLastFocusNodeWasSelected)
    return;

  gLastFocusNode = element;
  gLastFocusNodeWasSelected = mixed.oneElementSelected;

  var toolbar = document.getElementById("structToolbar");
  if (!toolbar) return;
  var childNodes = toolbar.childNodes;
  var childNodesLength = childNodes.length;
  // We need to leave the <label> to flex the buttons to the left
  // so, don't remove the last child at position length - 1
  for (var i = childNodesLength - 2; i >= 0; i--) {
    toolbar.removeChild(childNodes.item(i));
  }

  toolbar.removeAttribute("label");

  if ( IsInHTMLSourceMode() ) {
    // we have destroyed the contents of the status bar and are
    // about to recreate it ; but we don't want to do that in
    // Source mode
    return;
  }

  // XXXXXX
  if (true)
  {
    UpdateRulers(element);
    if (!gScrollListener)
      gScrollListener        = GetCurrentEditor().document.addEventListener("scroll", UpdateRulerRequestListener, false);
    if (!gResizeListener)
      gResizeListener        = window.addEventListener("resize", UpdateRulerRequestListener, false);
    if (!gEditorUpdatedListener)
      gEditorUpdatedListener = GetCurrentEditor().document.addEventListener("editorViewUpdated", UpdateRulerRequestListener, false);
  }

  var bodyElement = GetBodyElement();
  var isFocusNode = true;
  var tmp;
  var tag = gLastFocusNode.nodeName.toLowerCase();

  toolbar.firstChild.removeAttribute("style");
  var button;
  do {
    tag = element.nodeName.toLowerCase();

    // <Kaze>
    //~ button = document.createElementNS(XUL_NS, "toolbarbutton");
    //~ button.setAttribute("label",   "<" + tag + ">");
    //~ button.setAttribute("value",   tag);
    // Let's add "id" and "class" attributes in the tag
    // and other attributes in the tooltip
    button = newStructToolbarButton(tag, element.attributes);
    // </Kaze>
    button.setAttribute("context", "structToolbarContext");
    button.className = "struct-button";

    toolbar.insertBefore(button, toolbar.firstChild);

    button.addEventListener("command", newCommandListener(element), false);
    button.addEventListener("contextmenu", newContextmenuListener(button, element), false);

    if (isFocusNode && oneElementSelected) {
      button.setAttribute("checked", "true");
      isFocusNode = false;
    }

    tmp = element;
    element = element.parentNode;

  } while (tmp != bodyElement);
}
//
// integrate Launchy in the site manager and in the pref dialog
function NsmEditLaunchyXml() {
	var AppPath = NsmPrefs.getCharPref("extEditor");
	NsmLaunchExe(AppPath, LaunchyContext.xmlfile);
}

/* // OO-test
function NsmConTextMenu (menuid) {
} */
// this should be in a [NsmConTextMenu] class definition
function NsmClearMenu(menuid) {
	var children = menuid.childNodes;
	for (var j = children.length-1; j >= 0; j--)
		menuid.removeChild(children[j]);
}
function NsmAppendLaunchyItem(menuid, AppRef, command) {
	var menuitem = document.createElement("menuitem");	
	menuitem.setAttribute("label", LaunchyContext.apps[AppRef].name);
	//~ menuitem.setAttribute("label", "Ouvrir avec " + LaunchyContext.apps[AppRef].name);
	menuitem.setAttribute("oncommand", command);
	if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
		menuitem.setAttribute("class", "menuitem-iconic");
		menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[AppRef].exe + "?size=16");
	}
	menuid.appendChild(menuitem);
}
function NsmAppendLaunchyItems(menuid, type) {
	for (var i = 0; LaunchyContext.apps.length > i; i++)
		if (LaunchyContext.apps[i].type == type)
			NsmAppendLaunchyItem(menuid, i, "NsmLaunch(" + i + ", '')");
}
function NsmAppendMenuItem(menuid, label, command) {
	var menuitem = document.createElement("menuitem");
	menuitem.setAttribute("label", label);
	menuitem.setAttribute("oncommand", command);
	menuid.appendChild(menuitem);
}
function NsmAppendMenuPopup(menuid, label, command) {
	//~ var menuitem = document.createElement("menuitem");
	var menu = document.createElement("menu");
	menu.setAttribute("label", label);
	menu.setAttribute("id", "NsmMenu"+label);
	
	var menupopup = document.createElement("menupopup");
	menupopup.setAttribute("id", "NsmPopup"+label);
	//~ menupopup.setAttribute("onpopupshowing", command);	
	if (label == "Ouvrir") {
		//~ NsmConTextOpen(menupopup);
		NsmAppendMenuItem(menupopup, "Nouvel onglet", "NsmEdit(true,'')");
		NsmAppendMenuItem(menupopup, "Nouvelle fentre", "NsmEdit(false,'')");
		NsmAppendMenuSeparator(menupopup);
		//~ NsmAppendLaunchyItems(menupopup, 7);
	} else if (label == "Navigateur") {
		//~ NsmConTextPreview(menupopup);
		NsmAppendMenuItem(menupopup, "Navigateur par dfaut", "NsmPreviewAppRef=-1; loadExternalURL()");
		NsmAppendMenuSeparator(menupopup);
		//~ NsmAppendLaunchyItems(menupopup, 1);
	}
	menuid.appendChild(menu);
	menu.appendChild(menupopup);
}
function NsmAppendMenuSeparator(menuid) {
	var menuseparator = document.createElement("menuseparator");
	menuid.appendChild(menuseparator);
}
// sitemanagerOverlay.xul (sitemanager context menu)
function NsmConText(menuid) {
	//~ if (!IsHiddenByFilter("html", item.url))
		//~ return;
	//~ NsmApps = LaunchyContext.apps;
	
	NsmClearMenu(menuid);
	var item = gFilteredItemsArray[GetSelectedItem(gDialog.SiteTree)];
	if (item.isContainer) {
		NsmAppendLaunchyItems(menuid, 6);
	} else if (!IsHiddenByFilter("html", item.url)) {
		//~ NsmAppendMenuItem(menuid, "Ouvrir", "openFile(event)");
		NsmAppendMenuItem(menuid, "New tab", "NsmEdit(true,'')");
		NsmAppendMenuItem(menuid, "New window", "NsmEdit(false,'')");
		try {
			if (NsmPrefs.getBoolPref("ShowEditors")) {
				NsmAppendMenuSeparator(menuid);
				NsmAppendLaunchyItems(menuid, 7);
			}
			if (NsmPrefs.getBoolPref("ShowBrowsers")) {
				NsmAppendMenuSeparator(menuid);
				NsmAppendLaunchyItems(menuid, 1);
			}
		} catch (e) {};
		//~ NsmAppendMenuPopup(menuid, "Ouvrir", "NsmConTextOpen(this)");
		//~ NsmAppendMenuPopup(menuid, "Navigateur", "NsmConTextPreview(this)");
	} else if (!IsHiddenByFilter("images", item.url)) {
		NsmAppendLaunchyItems(menuid, 8);
	} else {
		NsmAppendLaunchyItems(menuid, 7);
	}
	
	if (item.level > 0) {
		NsmAppendMenuSeparator(menuid);
		NsmAppendMenuItem(menuid, "Rename", "Rename()");
		NsmAppendMenuItem(menuid, "Remove", "RemoveFileOrDir()");
	}
	NsmAppendMenuSeparator(menuid);
	NsmAppendMenuItem(menuid, "NsmConText...", "openPrefDialog()");
/*	
	//~ var menuitem = document.getElementById("NsmDefaultAction");
	//~ menuitem.setAttribute("class", "menuitem-iconic");
	//~ menuitem.setAttribute("image", iconPath);
	//~ alert(iconPath);
	//~ menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
	//~ menuitem.setAttribute("icon", icon);
}*/
}
function NsmConTextPreview(menuid) {
	//~ NsmClearMenu(menuid);
	NsmAppendMenuItem(menuid, "Navigateur par dfaut", "NsmPreviewAppRef=-1; loadExternalURL()");
	NsmAppendMenuSeparator(menuid);
	NsmAppendLaunchyItems(menuid, 1);
}
function NsmConTextOpen(menuid) {
	//~ NsmClearMenu(menuid);
	NsmAppendMenuItem(menuid, "Nouvel onglet", "NsmEdit(true,'')");
	NsmAppendMenuItem(menuid, "Nouvelle fentre", "NsmEdit(false,'')");
	NsmAppendMenuSeparator(menuid);
	NsmAppendLaunchyItems(menuid, 7);
}
function NsmLaunchyContext(menuid) {
	NsmClearMenu(menuid);

	var item = gFilteredItemsArray[GetSelectedItem(gDialog.SiteTree)];
	if (item.isContainer) {
		NsmAppendLaunchyItems(menuid, 6);
	} else {
		if (!IsHiddenByFilter("html", item.url)) {
			NsmAppendLaunchyItems(menuid, 1);
		}
		if (!IsHiddenByFilter("images", item.url)) {
			NsmAppendLaunchyItems(menuid, 8);
		} else {
			NsmAppendLaunchyItems(menuid, 7);
		}
	}

	var menuitem = document.createElement("menuitem");
	menuitem.setAttribute("label", "preferences...");
	menuitem.setAttribute("oncommand", "openDialog('chrome://launchy/content/options.xul', '', 'chrome,modal');");
	menuid.appendChild(menuitem);
/*	
	//~ var menuitem = document.getElementById("NsmDefaultAction");
	//~ menuitem.setAttribute("class", "menuitem-iconic");
	//~ menuitem.setAttribute("image", iconPath);
	//~ alert(iconPath);
	//~ menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
	//~ menuitem.setAttribute("icon", icon);
}*/
}
// NsmConText.xul (prefs window)
function addNsmPrefItems(menuid, type, pref) {
	NsmClearMenu(menuid);
	var path;
	var value;
	for (var i = 0; LaunchyContext.apps.length > i; i++)
		if (LaunchyContext.apps[i].type == type) {
			path = LaunchyContext.apps[i].exe.replace(/\\/g, "\\\\");
			value = "document.getElementById('" + pref + "').value = '" + path +"';";
			NsmAppendLaunchyItem(menuid, i, value);
		}
}
// editorPreviewOverlay.xul (navigator preview)
function addNsmPreviews(menuid, type) {
	NsmClearMenu(menuid);
	for (var i = 0; LaunchyContext.apps.length > i; i++)
		if (LaunchyContext.apps[i].type == type)
			NsmAppendLaunchyItem(menuid, i, "NsmPreviewAppRef=(" + i + ")");
}

//
// Using jslib to get Nvu's root and profile dirs (to use with PortableNvu)
const NVU_ROOT_PATH = NvuLocalDir("AChrom").parent.path;
const NVU_PROFILE_PATH = NvuLocalDir("PrefD").path;
function NvuLocalDir(dirId) {
	// taken from jslib
	const JS_FILESYSTEM_LOADED = true;
	const JS_FILESYSTEM_FILE   = "filesystem.js";
	const JS_FS_LOCAL_CID      = "@mozilla.org/file/local;1";
	const JS_FS_DIR_CID        = "@mozilla.org/file/directory_service;1";
	const JS_FS_NETWORK_CID    = '@mozilla.org/network/standard-url;1';
	const JS_FS_URL_COMP       = "nsIURL";
	const JS_FS_I_LOCAL_FILE   = "nsILocalFile";
	const JS_FS_DIR_I_PROPS    = "nsIProperties";
	const JS_FS_INIT_W_PATH    = "initWithPath";
	const JS_FS_CHROME_DIR     = "AChrom";		// /usr/src/mozilla/dist/bin/chrome
	const JS_FS_USR_DEFAULT    = "DefProfRt";	// /root/.mozilla
	const JS_FS_PREF_DIR       = "PrefD";		// /root/.mozilla/Default User/k1m30xaf.slt
	const JS_FS_OK             = true;
	const JS_FS_File_Path      = new Components.Constructor (JS_FS_LOCAL_CID, JS_FS_I_LOCAL_FILE, JS_FS_INIT_W_PATH);
	const JS_FS_Dir            = new Components.Constructor (JS_FS_DIR_CID, JS_FS_DIR_I_PROPS);
	const JS_FS_URL            = new Components.Constructor (JS_FS_NETWORK_CID, JS_FS_URL_COMP);
	// getting the requested dir
	var rvService = Components.classes[JS_FS_DIR_CID].getService(Components.interfaces.nsIProperties)
	var path = rvService.get(dirId, Components.interfaces.nsIFile).path;
	var dir = new JS_FS_File_Path(path);
	return dir;
}
// Using Launchy FileIO class to launch apps 
var NsmFileIO = {

	localfileCID : '@mozilla.org/file/local;1',
	localfileIID : Components.interfaces.nsILocalFile,

	urlCID : '@mozilla.org/network/standard-url;1',
	urlIID : Components.interfaces.nsIURL,
	
	processCID : '@mozilla.org/process/util;1',
	processIID : Components.interfaces.nsIProcess,
	
	urlToPath : function(data) {
		try {
			const hasUnescape = (typeof(unescape)=="function");
			var uri = Components.classes[this.urlCID].createInstance(this.urlIID);
			uri.spec = data;
			var file = Components.classes[this.localfileCID].createInstance(this.localfileIID);
			try {
				file.initWithPath(uri.path);
				data = hasUnescape ? unescape(file.path) : file.path;
			} catch (e) {
				// foobar!
			}
			try {
				file.initWithPath(uri.path.replace(/^\//,"").replace(/\//g,"\\"));
				data = hasUnescape ? unescape(file.path) : file.path;
			} catch (e) {
				// foobar!
			}
		}
		catch(e) {
			// foobar!
		}
		return data;
	},

	run : function(file, data) {
		try {
			var len = 0;
			if (data)
				len = data.length;
			else
				data = null;

			var lfile = Components.classes[this.localfileCID].createInstance(this.localfileIID);
			lfile.initWithPath(file);

			if (lfile.isFile() && lfile.isExecutable()) {
				try {
					var process = Components.classes[this.processCID].createInstance(this.processIID);
					process.init(lfile);
					data = process.run(false, data, len);
				} catch (e) {
					// foobar!
				}
			}
		}
		catch(e) {
			// foobar!
		}
		return data;
	}
}
function NsmLaunchExe(AppPath, FileURL) {
	AppPath = AppPath.replace(/^%NvuRoot%/, NVU_ROOT_PATH);
	AppPath = AppPath.replace(/^%NvuProfile%/, NVU_PROFILE_PATH);
	var success = 1;
	
	// only works on local files !
	try {
		var args = new Array();
		args[0] = NsmFileIO.urlToPath(FileURL);
		//~ var re = /^file:/i ;
		//~ if (re.test(FileURL))
			//~ args[0] = NsmFileIO.urlToPath(FileURL);
		//~ else
			//~ args[0] = FileURL;
		NsmFileIO.run(AppPath, args);
		//~ DEBUG ( "app launched" );
	} catch (e) {
		success = 0;
	};
	return success;
}
function NsmLaunch(LaunchyAppRef, FileURL) {
	if (FileURL.length > 0)
		url = FileURL;
	else
		url = gFilteredItemsArray[GetSelectedItem(gDialog.SiteTree)].url;
		
	var success = 1;
	var args = new Array();
	if (LaunchyContext.apps[LaunchyAppRef].args.length > 0)
		args[0] = LaunchyContext.apps[LaunchyAppRef].args;
	args[args.length] = NsmFileIO.urlToPath(url);
	
	var AppPath = LaunchyContext.apps[LaunchyAppRef].exe;
	AppPath = AppPath.replace(/^%NvuRoot%/, NVU_ROOT_PATH);
	AppPath = AppPath.replace(/^%NvuProfile%/, NVU_PROFILE_PATH);
	// only works on local files !
	//~ try {
		NsmFileIO.run(AppPath, args);
		//~ DEBUG ( "app launched" );
	//~ } catch (e) {
		success = 0;
	//~ };
	return success;
}
