/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 * Henrik Gemal http://gemal.dk
 *
 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var LaunchyShared = function() {}

LaunchyShared.prototype.isLinkType = function(linktype, link) {
	link = link.toString();
	try {
		var protocol = link.substr(0, linktype.length);
		return protocol.toLowerCase() == linktype;
	} catch(e) {
		launchyShared.trace("Unable to get protocol from " + link);
		return false;
	}
}

// Dump message to the JavaScript console but only if you're a developer
LaunchyShared.prototype.trace = function(t, f) {
	if (f || (typeof(LaunchyContext) != "undefined" && typeof(LaunchyContext.dev) != "undefined" && LaunchyContext.dev)) {
		var CONSOLE_SERVICE = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
		try {
			CONSOLE_SERVICE.logStringMessage("Launchy: " + t);
		} catch(ex) {
			CONSOLE_SERVICE.logStringMessage("Launchy: Unable to log message. Type was <" + typeof(t) + ">");
		}
	}
}

LaunchyShared.prototype.openURL = function(aURL) {
	if (navigator.vendor != "Thunderbird") {
		var navWindow;
		try {
			var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
			navWindow = wm.getMostRecentWindow("navigator:browser");
		}	catch (ex) {
		}

		if (navWindow) {
			if ("delayedOpenTab" in navWindow)
				navWindow.delayedOpenTab(aURL);
			else if ("loadURI" in navWindow)
				navWindow.loadURI(aURL);
			else
				navWindow.content.location.href = aURL;
		}	else {
			var ass = Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Components.interfaces.nsIAppShellService);
			w = ass.hiddenDOMWindow;
			w.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", aURL);
		}
	}	else {
		try {
			var messenger = Components.classes["@mozilla.org/messenger;1"].createInstance();
			messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
			messenger.launchExternalURL(aURL);
		} catch (ex) {
		}
	}
}

LaunchyShared.prototype.help = function(s) {
	var url = "http://gemal.dk/mozilla/launchy.html";
	if (s) {
		url += "#" + s;
	}
	launchyShared.openURL(url);
}

LaunchyShared.prototype.replaceInsert = function(t, i, v) {
	var result = t;
	var re = new RegExp("#" + i, "g");
	result = result.replace(re, v);
	return result;
}

// create the object
const launchyShared = new LaunchyShared();
